
//          Copyright Oliver Kowalke 2014.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#include <boost/context/detail/config.hpp>

#if !defined(BOOST_NO_CXX11_THREAD_LOCAL)
# include <boost/context/execution_context_v1.hpp>
#endif
#include <boost/context/execution_context_v2.hpp>
