
// Copyright (C) 2008-2018 Lorenzo Caminiti
// Distributed under the Boost Software License, Version 1.0 (see accompanying
// file LICENSE_1_0.txt or a copy at http://www.boost.org/LICENSE_1_0.txt).
// See: http://www.boost.org/doc/libs/release/libs/contract/doc/html/index.html

#include <boost/contract/core/config.hpp>
// For shared (default) and static lib only.
#ifndef BOOST_CONTRACT_HEADER_ONLY // This is not an #include guard.
    #define BOOST_CONTRACT_SOURCE
    #include <boost/contract/detail/inlined.hpp>
#endif // Else (header-only), make sure nothing is compiled here.

