
// Copyright (C) 2008-2018 Lorenzo Caminiti
// Distributed under the Boost Software License, Version 1.0 (see accompanying
// file LICENSE_1_0.txt or a copy at http://www.boost.org/LICENSE_1_0.txt).
// See: http://www.boost.org/doc/libs/release/libs/contract/doc/html/index.html

// Test no error if permissive even when static inv declared const.

#ifndef BOOST_CONTRACT_PERMISSIVE
    #error "build must define PERMISSIVE"
#endif
#include "static_const.hpp"

