/*=============================================================================
    Copyright (c) 2001-2011 Joel de Guzman

    Distributed under the Boost Software License, Version 1.0. (See accompanying 
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(SPIRIT_ACTION_JANUARY_07_2007_1233PM)
#define SPIRIT_ACTION_JANUARY_07_2007_1233PM

#if defined(_MSC_VER)
#pragma once
#endif

#include <boost/spirit/home/qi/action/action.hpp>

#endif
