
//          Copyright Oliver Kowalke 2009.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BIND_TO_PROCESSOR_H
#define BIND_TO_PROCESSOR_H

void bind_to_processor( unsigned int n);

#endif // BIND_TO_PROCESSOR_H
