// Boost.Geometry (aka GGL, Generic Geometry Library)
// Unit Test

// Copyright (c) 2014, Oracle and/or its affiliates.

// Contributed and/or modified by Menelaos Karavelas, on behalf of Oracle

// Licensed under the Boost Software License version 1.0.
// http://www.boost.org/users/license.html

#ifndef BOOST_TEST_MODULE
#define BOOST_TEST_MODULE test_distance_all
#endif

#include "distance_pointlike_pointlike.cpp"
#include "distance_pointlike_linear.cpp"
#include "distance_pointlike_areal.cpp"
#include "distance_linear_linear.cpp"
#include "distance_linear_areal.cpp"
#include "distance_areal_areal.cpp"
