//////////////////////////////////////////////////////////////////////////////
//
// (C) Copyright Vicente J. Botet Escriba 2010.
// Distributed under the Boost
// Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or
// copy at http://www.boost.org/LICENSE_1_0.txt)
//
// See http://www.boost.org/libs/stm for documentation.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef BOOST_STOPWATCHES_HPP
#define BOOST_STOPWATCHES_HPP

//-----------------------------------------------------------------------------
#include <boost/chrono/stopwatches/simple_stopwatch.hpp>
#include <boost/chrono/stopwatches/reporters/stopwatch_reporter.hpp>
#include <boost/chrono/stopwatches/reporters/system_default_formatter.hpp>
#include <boost/chrono/stopwatches/reporters/process_default_formatter.hpp>
#include <boost/chrono/stopwatches/reporters/thread_default_formatter.hpp>
//-----------------------------------------------------------------------------

#endif // BOOST_STOPWATCHES_HPP
