// Boost.Geometry Index
// Test

// Copyright (c) 2011-2013 Adam Wulkiewicz, Lodz, Poland.

// Use, modification and distribution is subject to the Boost Software License,
// Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#ifndef GEOMETRY_TEST_GEOMETRY_INDEX_TEST_COMMON_HPP
#define GEOMETRY_TEST_GEOMETRY_INDEX_TEST_COMMON_HPP

#if defined(_MSC_VER)

//#pragma warning (disable : 4996) // deprecated functions
//#pragma warning (disable : 4100) // unreferenced formal parameter
//#pragma warning (disable : 4127) // conditional expression is constant

#endif // _MSC_VER

#include <boost/geometry.hpp>

#include <geometry_test_common.hpp>

namespace boost { namespace geometry { namespace index {}}}
namespace bgi = boost::geometry::index;

#endif // GEOMETRY_TEST_GEOMETRY_INDEX_TEST_COMMON_HPP
